import greenfoot.*;
import java.util.List;
import java.awt.Color;

public abstract class Scrollworld  extends World
{
    private int Width;
    private int Height;
    private int ScreenLeft=0;
    private int ScreenUp=0;
    private GreenfootImage Background;
    private boolean BackgroundMove=true;
    private int CellSize;

    /**
     * First 2 collumns: Width and Height of the whole World
     * 3rd & 4th collumns: Width and Height of the Screen, that should be shown
     * 5th collumn: CellSize
     * The Position on the left uppon corner is 0,0
     * Background Moves,too
     */
    public Scrollworld(int Width, int Height,int ScreenWidth,int ScreenHeight,int CellSize)
    {
        this(Width,Height,ScreenWidth,ScreenHeight,0,0,CellSize,true);
    }

    /**
     * First 2 collumns: Width and Height of the whole World
     * 3rd & 4th collumns: Width and Height of the Screen, that should be shown
     * 5th & 6th collumns: The Position on the left uppon corner
     * 7th collumn: CellSize
     * Background Moves,too;
     */
    public Scrollworld(int Width, int Height,int ScreenWidth,int ScreenHeight,int CellSize,boolean BackgroundMove)
    {
        this(Width,Height,ScreenWidth,ScreenHeight,0,0,CellSize,BackgroundMove);
    }

    /**
     * First 2 collumns: Width and Height of the whole World
     * 3rd & 4th collumns: Width and Height of the Screen, that should be shown
     * 5th & 6th collumns: The Position on the left uppon corner
     * 7th collumn: CellSize
     * Background Moves,too;
     */
    public Scrollworld(int Width, int Height,int ScreenWidth,int ScreenHeight,int ScreenLeft,int ScreenUp,int CellSize)
    {
        this(Width,Height,ScreenWidth,ScreenHeight,ScreenLeft,ScreenUp,CellSize,true);
    }

    /**
     * First 2 collumns: Width and Height of the whole World
     * 3rd & 4th collumns: Width and Height of the Screen, that should be shown
     * 5th & 6th collumns: The Position on the left uppon corner
     * 7th collumn: CellSize
     * 8th collumn: should the background move, too
     */
    public Scrollworld(int Width, int Height,int ScreenWidth, int ScreenHeight,int ScreenLeft,int ScreenUp, int CellSize, boolean BackgroundMove)
    {
        super(ScreenWidth, ScreenHeight, CellSize, false);
        if(ScreenWidth>Width)
            ScreenWidth=Width;
        if(ScreenHeight>Height)
            ScreenWidth=Height;
        this.Width=Width;
        this.Height=Height;
        this.BackgroundMove=BackgroundMove;
        this.CellSize=CellSize;
        if(BackgroundMove)
        {
            Background=new GreenfootImage(Width,Height);
            Background.setColor(Color.white);
            Background.fillRect(0,0,Width,Height);
        }
        ShowWorld(ScreenLeft, ScreenUp);
    }

    /** That You can Scroll the World with arrow keys */
    public void act()
    {
        if(Greenfoot.isKeyDown("left"))
            ShowWorld(getScreenLeft()-2,getScreenUp());
        if(Greenfoot.isKeyDown("right"))
            ShowWorld(getScreenLeft()+2,getScreenUp());
        if(Greenfoot.isKeyDown("up"))
            ShowWorld(getScreenLeft(),getScreenUp()-2);
        if(Greenfoot.isKeyDown("down"))
            ShowWorld(getScreenLeft(),getScreenUp()+2);
    }

    /** return a smaller part from the given Image, with the width "Width" and the height "Height" from the Position fromX, fromY, on the Image */
    public GreenfootImage ShowPartOfImage(GreenfootImage Image, int fromX, int fromY, int Width, int Height)
    {
        GreenfootImage I = new GreenfootImage(Width, Height);
        I.drawImage(Image,-fromX,-fromY);
        return I;
    }

    /** Shows a part of the world. Thats the Position on the left uppon corner */
    public void ShowWorld(int ShowLeft, int ShowRight)
    {
        int x = ShowLeft;
        int y = ShowRight;
        if(x<0)
            x=0;
        if(y<0)
            y=0;
        if(x>Width-super.getWidth())
            x=Width-super.getWidth();
        if(y>Height-super.getHeight())
            y=Height-super.getHeight();
        ScreenLeft=x;
        ScreenUp=y;
        List<ScrollObjects> S = getObjects(ScrollObjects.class);
        for(ScrollObjects s : S)
        {
            s.setLocation(s.getX(),s.getY());
        }
        if(BackgroundMove)
        {
            RightBackgroundPosition();
        }
    }

    /** Scales the world */
    public void scaleWorld(int width, int height)
    {
        if(width<getScreenWidth())
            width=getScreenWidth();
        if(height<getScreenHeight())
            height=getScreenHeight();
        double X = (double)(width)/(double)(getWidth());
        double Y = (double)(height)/(double)(getHeight());
            ShowWorld((int)(getScreenLeft()*X),(int)(getScreenUp()*Y));
            Width=width;
            Height=height;
        List<ScrollObjects> Scroller = getObjects(ScrollObjects.class);
        for(ScrollObjects S : Scroller)
        {
            int x = S.getX();
            int y = S.getY();
            S.setLocation((int)(X*(double)(x)),(int)(Y*(double)(y)));
        }
    }

    /** Creates a new "Scroll-Background" */
    public void newBackground(String Background)
    {
        newBackground(new GreenfootImage(Background));
    }

    /** Creates a new "Scroll-Background" */
    public void newBackground(GreenfootImage background)
    {
        GreenfootImage i=new GreenfootImage(getWidth(),getHeight());
        int x=getWidth()/background.getWidth();
        while(x>=0)
        {
            int y=getHeight()/background.getHeight();
            while(y>=0)
            {
                i.drawImage(background,x*background.getWidth(),y*background.getHeight());
                y--;
            }
            x--;
        }
        Background=i;
        RightBackgroundPosition();
    }

    /** Now the Background wont be Scrolled */
    public void DontScrollBackground()
    {
        BackgroundMove=false;
    }

    /** Now the Background will be Scrolled */
    public void ScrollBackground()
    {
        BackgroundMove=true;
        RightBackgroundPosition();
    }

    /** Add an Object to the World, that will be scrolled */
    public void addObject(ScrollObjects S, int x, int y)
    {
        super.addObject(S,0,0);
        S.setLocation(x,y);
    }

    /** Add an Object to the World, that wont be scrolled */
    public void addObject(NoScrollObjects A, int x, int y)
    {
        super.addObject(A,x,y);
    }

    /** returns the Position on the left of your screen */
    public int getScreenLeft()
    {
        return ScreenLeft;
    }

    /** returns the Position on the right of your screen */
    public int getScreenRight()
    {
        return ScreenLeft+super.getWidth()-1;
    }

    /** returns the Positionon the top of your screen */
    public int getScreenUp()
    {
        return ScreenUp;
    }

    /** returns the Positionon the bottom of your screen */
    public int getScreenDown()
    {
        return ScreenUp+super.getHeight()-1;
    }

    /** returns the width of your screen */
    public int getScreenWidth()
    {
        return super.getWidth();
    }

    /** returns the height of your screen */
    public int getScreenHeight()
    {
        return super.getHeight();
    }

    /** returns the Width of the whole Scrollworld */
    public int getWidth()
    {
        return Width;
    }

    /** returns the Height of the whole Scrollworld */
    public int getHeight()
    {
        return Height;
    }

    private int CellSize()
    {
        return CellSize;
    }

    private void RightBackgroundPosition()
    {
        setBackground(ShowPartOfImage(Background,getScreenLeft()*CellSize(),getScreenUp()*CellSize(),getScreenWidth(),getScreenHeight()));
    }
}
